"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.load = exports.statelessLoad = exports.initialState = void 0;

require("airbnb-js-shims");

var _fp = require("date-fns/fp");

var _minimalLispParser = require("./minimal-lisp-parser");

var _ruleDm = require("./rule-dm");

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { if (!(Symbol.iterator in Object(arr) || Object.prototype.toString.call(arr) === "[object Arguments]")) { return; } var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var initialState = {
  active: false,
  flipped: false,
  // only connected to resetCondition
  onCooldown: false,
  // only connected to cooldown
  lastFired: (0, _fp.addYears)(-100)(new Date()) // ttl
  // onExpired

};
exports.initialState = initialState;

var check = function check(conf, state, onExpired) {
  if (!state) return [true, state];
  var active = state.active,
      lastFired = state.lastFired,
      onCooldown = state.onCooldown,
      ttl = state.ttl;
  if (!active) return [true, state];
  var now = new Date();
  var cooldown = conf.cooldown;

  if ((0, _fp.isBefore)(now)(ttl)) {
    // rule has expired
    if (onExpired) onExpired();
    return [true, _objectSpread({}, state, {
      active: false
    })];
  }

  if (onCooldown && !!cooldown) {
    var cooledDown = (0, _fp.isBefore)(now)((0, _fp.addSeconds)(cooldown)(lastFired));
    if (cooledDown) return [false, _objectSpread({}, state, {
      onCooldown: false
    })]; // continue
  }

  return [false, state]; // continue
};

var statelessLoad = function statelessLoad() {
  var conf,
      _ref,
      customParser,
      _ref$parserOptions,
      pOptions,
      parserPatcher,
      _ref$vars,
      vs,
      validation,
      onLoad,
      _conf$active,
      active,
      _conf$ttl,
      ttlStr,
      ttl,
      beginState,
      parser,
      run,
      _args2 = arguments;

  return regeneratorRuntime.async(function statelessLoad$(_context2) {
    while (1) {
      switch (_context2.prev = _context2.next) {
        case 0:
          conf = _args2.length > 0 && _args2[0] !== undefined ? _args2[0] : {};
          _ref = _args2.length > 1 && _args2[1] !== undefined ? _args2[1] : {}, customParser = _ref.customParser, _ref$parserOptions = _ref.parserOptions, pOptions = _ref$parserOptions === void 0 ? {} : _ref$parserOptions, parserPatcher = _ref.parserPatcher, _ref$vars = _ref.vars, vs = _ref$vars === void 0 ? {} : _ref$vars;
          validation = _ruleDm.Rule.validate(conf);

          if (!validation.error) {
            _context2.next = 5;
            break;
          }

          throw validation.error;

        case 5:
          onLoad = conf.onLoad, _conf$active = conf.active, active = _conf$active === void 0 ? false : _conf$active, _conf$ttl = conf.ttl, ttlStr = _conf$ttl === void 0 ? null : _conf$ttl;
          ttl = ttlStr ? new Date(ttlStr) : (0, _fp.addYears)(100)(new Date());
          beginState = _objectSpread({}, initialState, {
            active: active,
            ttl: ttl
          });

          if (!active) {
            _context2.next = 16;
            break;
          }

          parser = customParser || (0, _minimalLispParser.functionalParserWithVars)(vs, pOptions);

          if (!onLoad) {
            _context2.next = 15;
            break;
          }

          _context2.next = 13;
          return regeneratorRuntime.awrap((0, _minimalLispParser.asyncBlockEvaluator)(parser, onLoad, parserPatcher));

        case 13:
          _context2.next = 16;
          break;

        case 15:
          console.warn('Rule has no onLoad:', conf.id || conf.rid || conf.name || 'Noname');

        case 16:
          run = function run() {
            var state,
                _ref2,
                _ref2$reuseParser,
                reuseParser,
                _ref2$parserOptions,
                parserOptions,
                _ref2$vars,
                vars,
                onExpired,
                _check,
                _check2,
                done,
                maybeNewState,
                states,
                process,
                resetCondition,
                condition,
                resetActions,
                actions,
                cooldown,
                runParser,
                _conditionsMet,
                _states,
                flipped,
                onCooldown,
                conditionsMet,
                _args = arguments;

            return regeneratorRuntime.async(function run$(_context) {
              while (1) {
                switch (_context.prev = _context.next) {
                  case 0:
                    state = _args.length > 0 && _args[0] !== undefined ? _args[0] : {};
                    _ref2 = _args.length > 1 && _args[1] !== undefined ? _args[1] : {}, _ref2$reuseParser = _ref2.reuseParser, reuseParser = _ref2$reuseParser === void 0 ? false : _ref2$reuseParser, _ref2$parserOptions = _ref2.parserOptions, parserOptions = _ref2$parserOptions === void 0 ? {} : _ref2$parserOptions, _ref2$vars = _ref2.vars, vars = _ref2$vars === void 0 ? {} : _ref2$vars, onExpired = _ref2.onExpired;
                    // console.log('run:', id, vars, parserOptions, state);
                    _check = check(conf, state, onExpired), _check2 = _slicedToArray(_check, 2), done = _check2[0], maybeNewState = _check2[1];

                    if (!done) {
                      _context.next = 5;
                      break;
                    }

                    return _context.abrupt("return", [maybeNewState, undefined]);

                  case 5:
                    states = maybeNewState;
                    process = conf.process, resetCondition = conf.resetCondition, condition = conf.condition, resetActions = conf.resetActions, actions = conf.actions, cooldown = conf.cooldown;
                    runParser = reuseParser && parser ? parser : (0, _minimalLispParser.functionalParserWithVars)(vars, parserOptions);

                    if (!process) {
                      _context.next = 11;
                      break;
                    }

                    _context.next = 11;
                    return regeneratorRuntime.awrap((0, _minimalLispParser.asyncBlockEvaluator)(runParser, process));

                  case 11:
                    if (!(states.flipped && !!resetCondition)) {
                      _context.next = 21;
                      break;
                    }

                    _conditionsMet = runParser.evaluate(resetCondition);

                    if (!_conditionsMet) {
                      _context.next = 20;
                      break;
                    }

                    states = _objectSpread({}, states, {
                      flipped: false
                    });
                    _context.t0 = states;
                    _context.next = 18;
                    return regeneratorRuntime.awrap(resetActions ? (0, _minimalLispParser.asyncBlockEvaluator)(runParser, resetActions) : undefined);

                  case 18:
                    _context.t1 = _context.sent;
                    return _context.abrupt("return", [_context.t0, _context.t1]);

                  case 20:
                    return _context.abrupt("return", [states, undefined]);

                  case 21:
                    _states = states, flipped = _states.flipped, onCooldown = _states.onCooldown;

                    if (!(flipped || onCooldown)) {
                      _context.next = 24;
                      break;
                    }

                    return _context.abrupt("return", [states, undefined]);

                  case 24:
                    conditionsMet = condition === undefined ? true : runParser.evaluate(condition);

                    if (!conditionsMet) {
                      _context.next = 32;
                      break;
                    }

                    states = _objectSpread({}, states, {
                      flipped: !!resetCondition,
                      onCooldown: !!cooldown,
                      lastFired: new Date()
                    });
                    _context.t2 = states;
                    _context.next = 30;
                    return regeneratorRuntime.awrap(actions ? (0, _minimalLispParser.asyncBlockEvaluator)(runParser, actions) : undefined);

                  case 30:
                    _context.t3 = _context.sent;
                    return _context.abrupt("return", [_context.t2, _context.t3]);

                  case 32:
                    return _context.abrupt("return", [states, undefined]);

                  case 33:
                  case "end":
                    return _context.stop();
                }
              }
            });
          };

          return _context2.abrupt("return", [beginState, run]);

        case 18:
        case "end":
          return _context2.stop();
      }
    }
  });
};

exports.statelessLoad = statelessLoad;

var load = function load() {
  var state,
      tuple,
      _tuple,
      _args4 = arguments;

  return regeneratorRuntime.async(function load$(_context4) {
    while (1) {
      switch (_context4.prev = _context4.next) {
        case 0:
          state = {};
          _context4.next = 3;
          return regeneratorRuntime.awrap(statelessLoad.apply(void 0, _args4));

        case 3:
          tuple = _context4.sent;
          _tuple = _slicedToArray(tuple, 1);
          state = _tuple[0];
          return _context4.abrupt("return", function _callee() {
            var _len,
                b,
                _key,
                runned,
                _runned,
                _args3 = arguments;

            return regeneratorRuntime.async(function _callee$(_context3) {
              while (1) {
                switch (_context3.prev = _context3.next) {
                  case 0:
                    for (_len = _args3.length, b = new Array(_len), _key = 0; _key < _len; _key++) {
                      b[_key] = _args3[_key];
                    }

                    _context3.next = 3;
                    return regeneratorRuntime.awrap(tuple[1].apply(tuple, [state].concat(b)));

                  case 3:
                    runned = _context3.sent;
                    _runned = _slicedToArray(runned, 1);
                    state = _runned[0];
                    return _context3.abrupt("return", runned[1]);

                  case 7:
                  case "end":
                    return _context3.stop();
                }
              }
            });
          });

        case 7:
        case "end":
          return _context4.stop();
      }
    }
  });
};

exports.load = load;