"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.asyncBlockEvaluator = exports.createAsyncEvaluator = exports.functionalParserWithVars = exports.withFunctional = exports.withVars = exports.minimalLispParser = exports.getOrSet = void 0;

var _minimalLisp = _interopRequireDefault(require("minimal-lisp"));

var _get = _interopRequireDefault(require("lodash/fp/get"));

var _set = _interopRequireDefault(require("lodash/set"));

var R = _interopRequireWildcard(require("ramda"));

var dateFns = _interopRequireWildcard(require("date-fns/fp"));

var _crossFetch = _interopRequireDefault(require("cross-fetch"));

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || _typeof2(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _typeof2(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof2 = function _typeof2(obj) { return typeof obj; }; } else { _typeof2 = function _typeof2(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof2(obj); }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

// import { createJispParser } from './jisp';
var getOrSet = function getOrSet(vars) {
  return function (path, x) {
    var result = x === undefined ? (0, _get["default"])(path, vars) : (0, _get["default"])(path, (0, _set["default"])(vars, path, x));
    console.log("[miniMAL parser].var ".concat(path, ":"), result);
    return result;
  };
};

exports.getOrSet = getOrSet;

var minimalLispParser = function minimalLispParser() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      env = _ref.env,
      _ref$envExtra = _ref.envExtra,
      envExtra = _ref$envExtra === void 0 ? {} : _ref$envExtra,
      _ref$keepJsEval = _ref.keepJsEval,
      keepJsEval = _ref$keepJsEval === void 0 ? false : _ref$keepJsEval,
      doLog = _ref.doLog;

  var parser = (0, _minimalLisp["default"])(env || _objectSpread({
    undefined: undefined,
    "typeof": function _typeof(a) {
      return _typeof2(a);
    },
    // renaming of miniMAL's 'type'
    '>': function _(a, b) {
      return a > b;
    },
    '<=': function _(a, b) {
      return a <= b;
    },
    '>=': function _(a, b) {
      return a >= b;
    },
    '==': function _(a, b) {
      return Object.is(a, b);
    },
    '!=': function _(a, b) {
      return !Object.is(a, b);
    },
    '===': function _(a, b) {
      return a === b;
    },
    '!==': function _(a, b) {
      return a !== b;
    },
    '%': function _(a, b) {
      return a % b;
    },
    get: _get["default"],
    Array: Array,
    Object: Object,
    String: String,
    Number: Number,
    Promise: Promise,
    Date: Date,
    Math: Math,
    setInterval: setInterval,
    setTimeout: setTimeout,
    parseInt: parseInt,
    parseFloat: parseFloat,
    Set: Set,
    Map: Map,
    RegExp: RegExp,
    fetch: _crossFetch["default"],
    console: console,
    log: console.log
  }, envExtra));

  if (!keepJsEval) {
    parser.js = function () {
      throw new Error('Permission denied');
    };
  }

  parser.evalWithLog = function () {
    // DEPRECATED!
    // console.log('[miniMAL parser].eval in:', ...a);
    var result = parser.eval.apply(parser, arguments); // console.log('[miniMAL parser].eval out:', result);

    return result;
  };

  parser.evaluate = function () {
    var _console;

    for (var _len = arguments.length, a = new Array(_len), _key = 0; _key < _len; _key++) {
      a[_key] = arguments[_key];
    }

    if (doLog) (_console = console).log.apply(_console, ['[miniMAL parser].eval in:'].concat(a));
    var result = parser.eval.apply(parser, a);
    if (doLog) console.log('[miniMAL parser].eval out:', result);
    return result;
  };

  return parser;
};

exports.minimalLispParser = minimalLispParser;

var withVars = function withVars() {
  var vars = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  return function (parser) {
    parser["var"] = getOrSet(vars); // console.log('miniMAL parser:', parser);

    return parser;
  };
};

exports.withVars = withVars;

var withFunctional = function withFunctional(parser) {
  R.forEachObjIndexed(function (func, name) {
    // if (name !== 'default') parser[name] = func;
    if (name !== 'default') parser["R.".concat(name)] = func;
  }, R);
  R.forEachObjIndexed(function (func, name) {
    parser["D.".concat(name)] = func;
  }, dateFns);
  return parser;
};

exports.withFunctional = withFunctional;

var functionalParserWithVars = function functionalParserWithVars() {
  var vars = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var parserOptions = arguments.length > 1 ? arguments[1] : undefined;
  return R.compose(withFunctional, withVars(vars), minimalLispParser)(parserOptions);
};

exports.functionalParserWithVars = functionalParserWithVars;

var createAsyncEvaluator = function createAsyncEvaluator(parser, parserPatcher) {
  return function _callee(cmd) {
    var promiseMap, orderedPairs, keys, vals, resolvedValues, result;
    return regeneratorRuntime.async(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            if (!(_typeof2(cmd) === 'object' && !Array.isArray(cmd))) {
              _context.next = 10;
              break;
            }

            promiseMap = R.mapObjIndexed(function (val, key) {
              if (parserPatcher) parserPatcher(parser, key); // const result = parser.evaluate(val);

              return new Promise(function (r) {
                return r(parser.evaluate(val));
              })["catch"](function (err) {
                console.warn('[miniMAL block parser]', val, '->', err);
                return undefined;
              }); // if (result && typeof result.then === 'function') {
              //   return result.catch(err => {
              //     console.warn('[miniMAL block parser]', val, '->', err);
              //     return undefined;
              //   });
              // }
              // return Promise.resolve(result);
            })(cmd);
            orderedPairs = R.toPairs(promiseMap);
            keys = R.map(R.prop(0))(orderedPairs);
            vals = R.map(R.prop(1))(orderedPairs);
            _context.next = 7;
            return regeneratorRuntime.awrap(Promise.all(vals));

          case 7:
            resolvedValues = _context.sent;
            R.addIndex(R.forEach)(function (key, ix) {
              return parser["var"](key, resolvedValues[ix]);
            })(keys);
            return _context.abrupt("return", R.zipObj(keys, resolvedValues));

          case 10:
            result = parser.evaluate(cmd);

            if (!(result && typeof result.then === 'function')) {
              _context.next = 13;
              break;
            }

            return _context.abrupt("return", result["catch"](function (err) {
              console.warn('[miniMAL block parser]', cmd, '->', err);
              return undefined;
            }));

          case 13:
            return _context.abrupt("return", result);

          case 14:
          case "end":
            return _context.stop();
        }
      }
    });
  };
};

exports.createAsyncEvaluator = createAsyncEvaluator;

var asyncBlockEvaluator = function asyncBlockEvaluator(parser) {
  var block,
      parserPatcher,
      asyncEval,
      recurser,
      _args3 = arguments;
  return regeneratorRuntime.async(function asyncBlockEvaluator$(_context3) {
    while (1) {
      switch (_context3.prev = _context3.next) {
        case 0:
          block = _args3.length > 1 && _args3[1] !== undefined ? _args3[1] : [];
          parserPatcher = _args3.length > 2 ? _args3[2] : undefined;
          asyncEval = createAsyncEvaluator(parser, parserPatcher);

          recurser = function recurser() {
            var items,
                result,
                evaluated,
                _args2 = arguments;
            return regeneratorRuntime.async(function recurser$(_context2) {
              while (1) {
                switch (_context2.prev = _context2.next) {
                  case 0:
                    items = _args2.length > 0 && _args2[0] !== undefined ? _args2[0] : [];
                    result = _args2.length > 1 && _args2[1] !== undefined ? _args2[1] : [];

                    if (!R.isEmpty(items)) {
                      _context2.next = 4;
                      break;
                    }

                    return _context2.abrupt("return", result);

                  case 4:
                    _context2.next = 6;
                    return regeneratorRuntime.awrap(asyncEval(items[0]));

                  case 6:
                    evaluated = _context2.sent;
                    return _context2.abrupt("return", recurser(R.tail(items), R.append(evaluated)(result)));

                  case 8:
                  case "end":
                    return _context2.stop();
                }
              }
            });
          };

          return _context3.abrupt("return", recurser(block));

        case 5:
        case "end":
          return _context3.stop();
      }
    }
  });
};

exports.asyncBlockEvaluator = asyncBlockEvaluator;